@@ -7,7 +7,7 @@ from time import time
 from typing import Any, Dict, List, Mapping, Optional, Union
 
 import numpy as np
-from numpy.random import RandomState
+from numpy.random import default_rng
 
 from cmdstanpy import _TMPDIR
 from cmdstanpy.utils import (
@@ -29,11 +29,31 @@ class Method(Enum):
     OPTIMIZE = auto()
     GENERATE_QUANTITIES = auto()
     VARIATIONAL = auto()
+    LAPLACE = auto()
+    PATHFINDER = auto()
 
     def __repr__(self) -> str:
         return '<%s.%s>' % (self.__class__.__name__, self.name)
 
 
+def positive_int(value: Any, name: str) -> None:
+    if value is not None:
+        if isinstance(value, (int, np.integer)):
+            if value <= 0:
+                raise ValueError(f'{name} must be greater than 0')
+        else:
+            raise ValueError(f'{name} must be of type int')
+
+
+def positive_float(value: Any, name: str) -> None:
+    if value is not None:
+        if isinstance(value, (int, float, np.floating)):
+            if value <= 0:
+                raise ValueError(f'{name} must be greater than 0')
+        else:
+            raise ValueError(f'{name} must be of type float')
+
+
 class SamplerArgs:
     """Arguments for the NUTS adaptive sampler."""
 
@@ -54,6 +74,7 @@ class SamplerArgs:
         adapt_metric_window: Optional[int] = None,
         adapt_step_size: Optional[int] = None,
         fixed_param: bool = False,
+        num_chains: int = 1,
     ) -> None:
         """Initialize object."""
         self.iter_warmup = iter_warmup
@@ -72,6 +93,7 @@ class SamplerArgs:
         self.adapt_step_size = adapt_step_size
         self.fixed_param = fixed_param
         self.diagnostic_file = None
+        self.num_chains = num_chains
 
     def validate(self, chains: Optional[int]) -> None:
         """
@@ -81,7 +103,7 @@ class SamplerArgs:
         * if file(s) for metric are supplied, check contents.
         * length of per-chain lists equals specified # of chains
         """
-        if not isinstance(chains, int) or chains < 1:
+        if not isinstance(chains, (int, np.integer)) or chains < 1:
             raise ValueError(
                 'Sampler expects number of chains to be greater than 0.'
             )
@@ -110,40 +132,33 @@ class SamplerArgs:
                 raise ValueError(msg)
 
         if self.iter_warmup is not None:
-            if self.iter_warmup < 0 or not isinstance(self.iter_warmup, int):
+            if self.iter_warmup < 0 or not isinstance(
+                self.iter_warmup, (int, np.integer)
+            ):
                 raise ValueError(
                     'Value for iter_warmup must be a non-negative integer,'
                     ' found {}.'.format(self.iter_warmup)
                 )
-            if self.iter_warmup > 0 and not self.adapt_engaged:
+            if self.iter_warmup == 0 and self.adapt_engaged:
                 raise ValueError(
-                    'Argument "adapt_engaged" is False, '
-                    'cannot specify warmup iterations.'
+                    'Must specify iter_warmup > 0 when adapt_engaged=True.'
                 )
         if self.iter_sampling is not None:
             if self.iter_sampling < 0 or not isinstance(
-                self.iter_sampling, int
+                self.iter_sampling, (int, np.integer)
             ):
                 raise ValueError(
                     'Argument "iter_sampling" must be a non-negative integer,'
                     ' found {}.'.format(self.iter_sampling)
                 )
-        if self.thin is not None:
-            if self.thin < 1 or not isinstance(self.thin, int):
-                raise ValueError(
-                    'Argument "thin" must be a positive integer,'
-                    'found {}.'.format(self.thin)
-                )
-        if self.max_treedepth is not None:
-            if self.max_treedepth < 1 or not isinstance(
-                self.max_treedepth, int
-            ):
-                raise ValueError(
-                    'Argument "max_treedepth" must be a positive integer,'
-                    ' found {}.'.format(self.max_treedepth)
-                )
+
+        positive_int(self.thin, 'thin')
+        positive_int(self.max_treedepth, 'max_treedepth')
+
         if self.step_size is not None:
-            if isinstance(self.step_size, (float, int)):
+            if isinstance(
+                self.step_size, (float, int, np.integer, np.floating)
+            ):
                 if self.step_size <= 0:
                     raise ValueError(
                         'Argument "step_size" must be > 0, '
@@ -178,7 +193,7 @@ class SamplerArgs:
                     else:
                         self.metric_type = 'dense_e'
                     self.metric_file = self.metric
-            elif isinstance(self.metric, Dict):
+            elif isinstance(self.metric, dict):
                 if 'inv_metric' not in self.metric:
                     raise ValueError(
                         'Entry "inv_metric" not found in metric dict.'
@@ -289,7 +304,7 @@ class SamplerArgs:
                 )
         if self.adapt_init_phase is not None:
             if self.adapt_init_phase < 0 or not isinstance(
-                self.adapt_init_phase, int
+                self.adapt_init_phase, (int, np.integer)
             ):
                 raise ValueError(
                     'Argument "adapt_init_phase" must be a non-negative '
@@ -297,7 +312,7 @@ class SamplerArgs:
                 )
         if self.adapt_metric_window is not None:
             if self.adapt_metric_window < 0 or not isinstance(
-                self.adapt_metric_window, int
+                self.adapt_metric_window, (int, np.integer)
             ):
                 raise ValueError(
                     'Argument "adapt_metric_window" must be a non-negative '
@@ -305,12 +320,13 @@ class SamplerArgs:
                 )
         if self.adapt_step_size is not None:
             if self.adapt_step_size < 0 or not isinstance(
-                self.adapt_step_size, int
+                self.adapt_step_size, (int, np.integer)
             ):
                 raise ValueError(
                     'Argument "adapt_step_size" must be a non-negative integer,'
                     'found {}'.format(self.adapt_step_size)
                 )
+        positive_int(self.num_chains, 'num_chains')
 
         if self.fixed_param and (
             self.max_treedepth is not None
@@ -333,13 +349,13 @@ class SamplerArgs:
         """
         cmd.append('method=sample')
         if self.iter_sampling is not None:
-            cmd.append('num_samples={}'.format(self.iter_sampling))
+            cmd.append(f'num_samples={self.iter_sampling}')
         if self.iter_warmup is not None:
-            cmd.append('num_warmup={}'.format(self.iter_warmup))
+            cmd.append(f'num_warmup={self.iter_warmup}')
         if self.save_warmup:
             cmd.append('save_warmup=1')
         if self.thin is not None:
-            cmd.append('thin={}'.format(self.thin))
+            cmd.append(f'thin={self.thin}')
         if self.fixed_param:
             cmd.append('algorithm=fixed_param')
             return cmd
@@ -347,32 +363,34 @@ class SamplerArgs:
             cmd.append('algorithm=hmc')
         if self.max_treedepth is not None:
             cmd.append('engine=nuts')
-            cmd.append('max_depth={}'.format(self.max_treedepth))
+            cmd.append(f'max_depth={self.max_treedepth}')
         if self.step_size is not None:
             if not isinstance(self.step_size, list):
-                cmd.append('stepsize={}'.format(self.step_size))
+                cmd.append(f'stepsize={self.step_size}')
             else:
-                cmd.append('stepsize={}'.format(self.step_size[idx]))
+                cmd.append(f'stepsize={self.step_size[idx]}')
         if self.metric is not None:
-            cmd.append('metric={}'.format(self.metric_type))
+            cmd.append(f'metric={self.metric_type}')
         if self.metric_file is not None:
             if not isinstance(self.metric_file, list):
-                cmd.append('metric_file={}'.format(self.metric_file))
+                cmd.append(f'metric_file={self.metric_file}')
             else:
-                cmd.append('metric_file={}'.format(self.metric_file[idx]))
+                cmd.append(f'metric_file={self.metric_file[idx]}')
         cmd.append('adapt')
         if self.adapt_engaged:
             cmd.append('engaged=1')
         else:
             cmd.append('engaged=0')
         if self.adapt_delta is not None:
-            cmd.append('delta={}'.format(self.adapt_delta))
+            cmd.append(f'delta={self.adapt_delta}')
         if self.adapt_init_phase is not None:
-            cmd.append('init_buffer={}'.format(self.adapt_init_phase))
+            cmd.append(f'init_buffer={self.adapt_init_phase}')
         if self.adapt_metric_window is not None:
-            cmd.append('window={}'.format(self.adapt_metric_window))
+            cmd.append(f'window={self.adapt_metric_window}')
         if self.adapt_step_size is not None:
             cmd.append('term_buffer={}'.format(self.adapt_step_size))
+        if self.num_chains > 1:
+            cmd.append('num_chains={}'.format(self.num_chains))
 
         return cmd
 
@@ -381,6 +399,15 @@ class OptimizeArgs:
     """Container for arguments for the optimizer."""
 
     OPTIMIZE_ALGOS = {'BFGS', 'bfgs', 'LBFGS', 'lbfgs', 'Newton', 'newton'}
+    bfgs_only = {
+        "init_alpha",
+        "tol_obj",
+        "tol_rel_obj",
+        "tol_grad",
+        "tol_rel_grad",
+        "tol_param",
+        "history_size",
+    }
 
     def __init__(
         self,
@@ -394,8 +421,8 @@ class OptimizeArgs:
         tol_rel_grad: Optional[float] = None,
         tol_param: Optional[float] = None,
         history_size: Optional[int] = None,
+        jacobian: bool = False,
     ) -> None:
-
         self.algorithm = algorithm or ""
         self.init_alpha = init_alpha
         self.iter = iter
@@ -406,11 +433,9 @@ class OptimizeArgs:
         self.tol_rel_grad = tol_rel_grad
         self.tol_param = tol_param
         self.history_size = history_size
-        self.thin = None
+        self.jacobian = jacobian
 
-    def validate(
-        self, chains: Optional[int] = None  # pylint: disable=unused-argument
-    ) -> None:
+    def validate(self, _chains: Optional[int] = None) -> None:
         """
         Check arguments correctness and consistency.
         """
@@ -421,116 +446,177 @@ class OptimizeArgs:
                 )
             )
 
-        if self.init_alpha is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'init_alpha requires that algorithm be set to bfgs or lbfgs'
-                )
-            if isinstance(self.init_alpha, float):
-                if self.init_alpha <= 0:
-                    raise ValueError('init_alpha must be greater than 0')
-            else:
-                raise ValueError('init_alpha must be type of float')
-
-        if self.iter is not None:
-            if isinstance(self.iter, int):
-                if self.iter < 0:
-                    raise ValueError('iter must be greater than 0')
-            else:
-                raise ValueError('iter must be type of int')
-
-        if self.tol_obj is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'tol_obj requires that algorithm be set to bfgs or lbfgs'
-                )
-            if isinstance(self.tol_obj, float):
-                if self.tol_obj <= 0:
-                    raise ValueError('tol_obj must be greater than 0')
-            else:
-                raise ValueError('tol_obj must be type of float')
-
-        if self.tol_rel_obj is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'tol_rel_obj requires that algorithm be set to bfgs'
-                    ' or lbfgs'
-                )
-            if isinstance(self.tol_rel_obj, float):
-                if self.tol_rel_obj <= 0:
-                    raise ValueError('tol_rel_obj must be greater than 0')
-            else:
-                raise ValueError('tol_rel_obj must be type of float')
-
-        if self.tol_grad is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'tol_grad requires that algorithm be set to bfgs or lbfgs'
-                )
-            if isinstance(self.tol_grad, float):
-                if self.tol_grad <= 0:
-                    raise ValueError('tol_grad must be greater than 0')
-            else:
-                raise ValueError('tol_grad must be type of float')
-
-        if self.tol_rel_grad is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'tol_rel_grad requires that algorithm be set to bfgs'
-                    ' or lbfgs'
-                )
-            if isinstance(self.tol_rel_grad, float):
-                if self.tol_rel_grad <= 0:
-                    raise ValueError('tol_rel_grad must be greater than 0')
-            else:
-                raise ValueError('tol_rel_grad must be type of float')
-
-        if self.tol_param is not None:
-            if self.algorithm.lower() not in {'lbfgs', 'bfgs'}:
-                raise ValueError(
-                    'tol_param requires that algorithm be set to bfgs or lbfgs'
-                )
-            if isinstance(self.tol_param, float):
-                if self.tol_param <= 0:
-                    raise ValueError('tol_param must be greater than 0')
-            else:
-                raise ValueError('tol_param must be type of float')
-
-        if self.history_size is not None:
-            if self.algorithm.lower() != 'lbfgs':
+        if self.algorithm.lower() not in {'bfgs', 'lbfgs'}:
+            for arg in self.bfgs_only:
+                if getattr(self, arg) is not None:
+                    raise ValueError(
+                        f'{arg} requires that algorithm be set to bfgs or lbfgs'
+                    )
+        if self.algorithm.lower() != 'lbfgs':
+            if self.history_size is not None:
                 raise ValueError(
                     'history_size requires that algorithm be set to lbfgs'
                 )
-            if isinstance(self.history_size, int):
-                if self.history_size < 0:
-                    raise ValueError('history_size must be greater than 0')
-            else:
-                raise ValueError('history_size must be type of int')
 
-    # pylint: disable=unused-argument
-    def compose(self, idx: int, cmd: List[str]) -> List[str]:
+        positive_float(self.init_alpha, 'init_alpha')
+        positive_int(self.iter, 'iter')
+        positive_float(self.tol_obj, 'tol_obj')
+        positive_float(self.tol_rel_obj, 'tol_rel_obj')
+        positive_float(self.tol_grad, 'tol_grad')
+        positive_float(self.tol_rel_grad, 'tol_rel_grad')
+        positive_float(self.tol_param, 'tol_param')
+        positive_int(self.history_size, 'history_size')
+
+    def compose(self, _idx: int, cmd: List[str]) -> List[str]:
         """compose command string for CmdStan for non-default arg values."""
         cmd.append('method=optimize')
         if self.algorithm:
-            cmd.append('algorithm={}'.format(self.algorithm.lower()))
+            cmd.append(f'algorithm={self.algorithm.lower()}')
         if self.init_alpha is not None:
-            cmd.append('init_alpha={}'.format(self.init_alpha))
+            cmd.append(f'init_alpha={self.init_alpha}')
         if self.tol_obj is not None:
-            cmd.append('tol_obj={}'.format(self.tol_obj))
+            cmd.append(f'tol_obj={self.tol_obj}')
         if self.tol_rel_obj is not None:
-            cmd.append('tol_rel_obj={}'.format(self.tol_rel_obj))
+            cmd.append(f'tol_rel_obj={self.tol_rel_obj}')
         if self.tol_grad is not None:
-            cmd.append('tol_grad={}'.format(self.tol_grad))
+            cmd.append(f'tol_grad={self.tol_grad}')
         if self.tol_rel_grad is not None:
-            cmd.append('tol_rel_grad={}'.format(self.tol_rel_grad))
+            cmd.append(f'tol_rel_grad={self.tol_rel_grad}')
         if self.tol_param is not None:
-            cmd.append('tol_param={}'.format(self.tol_param))
+            cmd.append(f'tol_param={self.tol_param}')
         if self.history_size is not None:
-            cmd.append('history_size={}'.format(self.history_size))
+            cmd.append(f'history_size={self.history_size}')
         if self.iter is not None:
-            cmd.append('iter={}'.format(self.iter))
+            cmd.append(f'iter={self.iter}')
         if self.save_iterations:
             cmd.append('save_iterations=1')
+        if self.jacobian:
+            cmd.append("jacobian=1")
+        return cmd
+
+
+class LaplaceArgs:
+    """Arguments needed for laplace method."""
+
+    def __init__(
+        self, mode: str, draws: Optional[int] = None, jacobian: bool = True
+    ) -> None:
+        self.mode = mode
+        self.jacobian = jacobian
+        self.draws = draws
+
+    def validate(self, _chains: Optional[int] = None) -> None:
+        """Check arguments correctness and consistency."""
+        if not os.path.exists(self.mode):
+            raise ValueError(f'Invalid path for mode file: {self.mode}')
+        positive_int(self.draws, 'draws')
+
+    def compose(self, _idx: int, cmd: List[str]) -> List[str]:
+        """compose command string for CmdStan for non-default arg values."""
+        cmd.append('method=laplace')
+        cmd.append(f'mode={self.mode}')
+        if self.draws:
+            cmd.append(f'draws={self.draws}')
+        if not self.jacobian:
+            cmd.append("jacobian=0")
+        return cmd
+
+
+class PathfinderArgs:
+    """Container for arguments for Pathfinder."""
+
+    def __init__(
+        self,
+        init_alpha: Optional[float] = None,
+        tol_obj: Optional[float] = None,
+        tol_rel_obj: Optional[float] = None,
+        tol_grad: Optional[float] = None,
+        tol_rel_grad: Optional[float] = None,
+        tol_param: Optional[float] = None,
+        history_size: Optional[int] = None,
+        num_psis_draws: Optional[int] = None,
+        num_paths: Optional[int] = None,
+        max_lbfgs_iters: Optional[int] = None,
+        num_draws: Optional[int] = None,
+        num_elbo_draws: Optional[int] = None,
+        save_single_paths: bool = False,
+        psis_resample: bool = True,
+        calculate_lp: bool = True,
+    ) -> None:
+        self.init_alpha = init_alpha
+        self.tol_obj = tol_obj
+        self.tol_rel_obj = tol_rel_obj
+        self.tol_grad = tol_grad
+        self.tol_rel_grad = tol_rel_grad
+        self.tol_param = tol_param
+        self.history_size = history_size
+
+        self.num_psis_draws = num_psis_draws
+        self.num_paths = num_paths
+        self.max_lbfgs_iters = max_lbfgs_iters
+        self.num_draws = num_draws
+        self.num_elbo_draws = num_elbo_draws
+
+        self.save_single_paths = save_single_paths
+        self.psis_resample = psis_resample
+        self.calculate_lp = calculate_lp
+
+    def validate(self, _chains: Optional[int] = None) -> None:
+        """
+        Check arguments correctness and consistency.
+        """
+        positive_float(self.init_alpha, 'init_alpha')
+        positive_float(self.tol_obj, 'tol_obj')
+        positive_float(self.tol_rel_obj, 'tol_rel_obj')
+        positive_float(self.tol_grad, 'tol_grad')
+        positive_float(self.tol_rel_grad, 'tol_rel_grad')
+        positive_float(self.tol_param, 'tol_param')
+        positive_int(self.history_size, 'history_size')
+
+        positive_int(self.num_psis_draws, 'num_psis_draws')
+        positive_int(self.num_paths, 'num_paths')
+        positive_int(self.max_lbfgs_iters, 'max_lbfgs_iters')
+        positive_int(self.num_draws, 'num_draws')
+        positive_int(self.num_elbo_draws, 'num_elbo_draws')
+
+    def compose(self, _idx: int, cmd: List[str]) -> List[str]:
+        """compose command string for CmdStan for non-default arg values."""
+        cmd.append('method=pathfinder')
+
+        if self.init_alpha is not None:
+            cmd.append(f'init_alpha={self.init_alpha}')
+        if self.tol_obj is not None:
+            cmd.append(f'tol_obj={self.tol_obj}')
+        if self.tol_rel_obj is not None:
+            cmd.append(f'tol_rel_obj={self.tol_rel_obj}')
+        if self.tol_grad is not None:
+            cmd.append(f'tol_grad={self.tol_grad}')
+        if self.tol_rel_grad is not None:
+            cmd.append(f'tol_rel_grad={self.tol_rel_grad}')
+        if self.tol_param is not None:
+            cmd.append(f'tol_param={self.tol_param}')
+        if self.history_size is not None:
+            cmd.append(f'history_size={self.history_size}')
+
+        if self.num_psis_draws is not None:
+            cmd.append(f'num_psis_draws={self.num_psis_draws}')
+        if self.num_paths is not None:
+            cmd.append(f'num_paths={self.num_paths}')
+        if self.max_lbfgs_iters is not None:
+            cmd.append(f'max_lbfgs_iters={self.max_lbfgs_iters}')
+        if self.num_draws is not None:
+            cmd.append(f'num_draws={self.num_draws}')
+        if self.num_elbo_draws is not None:
+            cmd.append(f'num_elbo_draws={self.num_elbo_draws}')
+
+        if self.save_single_paths:
+            cmd.append('save_single_paths=1')
+
+        if not self.psis_resample:
+            cmd.append('psis_resample=0')
+
+        if not self.calculate_lp:
+            cmd.append('calculate_lp=0')
 
         return cmd
 
@@ -561,7 +647,7 @@ class GenerateQuantitiesArgs:
         Compose CmdStan command for method-specific non-default arguments.
         """
         cmd.append('method=generate_quantities')
-        cmd.append('fitted_params={}'.format(self.sample_csv_files[idx]))
+        cmd.append(f'fitted_params={self.sample_csv_files[idx]}')
         return cmd
 
 
@@ -609,58 +695,14 @@ class VariationalArgs:
                     ', '.join(self.VARIATIONAL_ALGOS)
                 )
             )
-        if self.iter is not None:
-            if self.iter < 1 or not isinstance(self.iter, int):
-                raise ValueError(
-                    'iter must be a positive integer,'
-                    ' found {}'.format(self.iter)
-                )
-        if self.grad_samples is not None:
-            if self.grad_samples < 1 or not isinstance(self.grad_samples, int):
-                raise ValueError(
-                    'grad_samples must be a positive integer,'
-                    ' found {}'.format(self.grad_samples)
-                )
-        if self.elbo_samples is not None:
-            if self.elbo_samples < 1 or not isinstance(self.elbo_samples, int):
-                raise ValueError(
-                    'elbo_samples must be a positive integer,'
-                    ' found {}'.format(self.elbo_samples)
-                )
-        if self.eta is not None:
-            if self.eta < 0 or not isinstance(self.eta, (int, float)):
-                raise ValueError(
-                    'eta must be a non-negative number,'
-                    ' found {}'.format(self.eta)
-                )
-        if self.adapt_iter is not None:
-            if self.adapt_iter < 1 or not isinstance(self.adapt_iter, int):
-                raise ValueError(
-                    'adapt_iter must be a positive integer,'
-                    ' found {}'.format(self.adapt_iter)
-                )
-        if self.tol_rel_obj is not None:
-            if self.tol_rel_obj <= 0 or not isinstance(
-                self.tol_rel_obj, (int, float)
-            ):
-                raise ValueError(
-                    'tol_rel_obj must be a positive number,'
-                    ' found {}'.format(self.tol_rel_obj)
-                )
-        if self.eval_elbo is not None:
-            if self.eval_elbo < 1 or not isinstance(self.eval_elbo, int):
-                raise ValueError(
-                    'eval_elbo must be a positive integer,'
-                    ' found {}'.format(self.eval_elbo)
-                )
-        if self.output_samples is not None:
-            if self.output_samples < 1 or not isinstance(
-                self.output_samples, int
-            ):
-                raise ValueError(
-                    'output_samples must be a positive integer,'
-                    ' found {}'.format(self.output_samples)
-                )
+        positive_int(self.iter, 'iter')
+        positive_int(self.grad_samples, 'grad_samples')
+        positive_int(self.elbo_samples, 'elbo_samples')
+        positive_float(self.eta, 'eta')
+        positive_int(self.adapt_iter, 'adapt_iter')
+        positive_float(self.tol_rel_obj, 'tol_rel_obj')
+        positive_int(self.eval_elbo, 'eval_elbo')
+        positive_int(self.output_samples, 'output_samples')
 
     # pylint: disable=unused-argument
     def compose(self, idx: int, cmd: List[str]) -> List[str]:
@@ -669,28 +711,28 @@ class VariationalArgs:
         """
         cmd.append('method=variational')
         if self.algorithm is not None:
-            cmd.append('algorithm={}'.format(self.algorithm))
+            cmd.append(f'algorithm={self.algorithm}')
         if self.iter is not None:
-            cmd.append('iter={}'.format(self.iter))
+            cmd.append(f'iter={self.iter}')
         if self.grad_samples is not None:
-            cmd.append('grad_samples={}'.format(self.grad_samples))
+            cmd.append(f'grad_samples={self.grad_samples}')
         if self.elbo_samples is not None:
-            cmd.append('elbo_samples={}'.format(self.elbo_samples))
+            cmd.append(f'elbo_samples={self.elbo_samples}')
         if self.eta is not None:
-            cmd.append('eta={}'.format(self.eta))
+            cmd.append(f'eta={self.eta}')
         cmd.append('adapt')
         if self.adapt_engaged:
             cmd.append('engaged=1')
             if self.adapt_iter is not None:
-                cmd.append('iter={}'.format(self.adapt_iter))
+                cmd.append(f'iter={self.adapt_iter}')
         else:
             cmd.append('engaged=0')
         if self.tol_rel_obj is not None:
-            cmd.append('tol_rel_obj={}'.format(self.tol_rel_obj))
+            cmd.append(f'tol_rel_obj={self.tol_rel_obj}')
         if self.eval_elbo is not None:
-            cmd.append('eval_elbo={}'.format(self.eval_elbo))
+            cmd.append(f'eval_elbo={self.eval_elbo}')
         if self.output_samples is not None:
-            cmd.append('output_samples={}'.format(self.output_samples))
+            cmd.append(f'output_samples={self.output_samples}')
         return cmd
 
 
@@ -707,7 +749,12 @@ class CmdStanArgs:
         model_exe: OptionalPath,
         chain_ids: Optional[List[int]],
         method_args: Union[
-            SamplerArgs, OptimizeArgs, GenerateQuantitiesArgs, VariationalArgs
+            SamplerArgs,
+            OptimizeArgs,
+            GenerateQuantitiesArgs,
+            VariationalArgs,
+            LaplaceArgs,
+            PathfinderArgs,
         ],
         data: Union[Mapping[str, Any], str, None] = None,
         seed: Union[int, List[int], None] = None,
@@ -739,6 +786,14 @@ class CmdStanArgs:
             self.method = Method.GENERATE_QUANTITIES
         elif isinstance(method_args, VariationalArgs):
             self.method = Method.VARIATIONAL
+        elif isinstance(method_args, LaplaceArgs):
+            self.method = Method.LAPLACE
+        elif isinstance(method_args, PathfinderArgs):
+            self.method = Method.PATHFINDER
+        else:
+            raise ValueError(
+                'Unsupported method args type: {}'.format(type(method_args))
+            )
         self.method_args.validate(len(chain_ids) if chain_ids else None)
         self.validate()
 
@@ -792,7 +847,10 @@ class CmdStanArgs:
                     ' cannot write to dir: {}.'.format(self.output_dir)
                 ) from exc
         if self.refresh is not None:
-            if not isinstance(self.refresh, int) or self.refresh < 1:
+            if (
+                not isinstance(self.refresh, (int, np.integer))
+                or self.refresh < 1
+            ):
                 raise ValueError(
                     'Argument "refresh" must be a positive integer value, '
                     'found {}.'.format(self.refresh)
@@ -800,7 +858,7 @@ class CmdStanArgs:
 
         if self.sig_figs is not None:
             if (
-                not isinstance(self.sig_figs, int)
+                not isinstance(self.sig_figs, (int, np.integer))
                 or self.sig_figs < 1
                 or self.sig_figs > 18
             ):
@@ -819,16 +877,16 @@ class CmdStanArgs:
                 )
 
         if self.seed is None:
-            rng = RandomState()
-            self.seed = rng.randint(1, 99999 + 1)
+            rng = default_rng()
+            self.seed = rng.integers(low=1, high=99999, size=1).item()
         else:
-            if not isinstance(self.seed, (int, list)):
+            if not isinstance(self.seed, (int, list, np.integer)):
                 raise ValueError(
                     'Argument "seed" must be an integer between '
                     '0 and 2**32-1, found {}.'.format(self.seed)
                 )
-            if isinstance(self.seed, int):
-                if self.seed < 0 or self.seed > 2 ** 32 - 1:
+            if isinstance(self.seed, (int, np.integer)):
+                if self.seed < 0 or self.seed > 2**32 - 1:
                     raise ValueError(
                         'Argument "seed" must be an integer between '
                         '0 and 2**32-1, found {}.'.format(self.seed)
@@ -847,7 +905,7 @@ class CmdStanArgs:
                         )
                     )
                 for seed in self.seed:
-                    if seed < 0 or seed > 2 ** 32 - 1:
+                    if seed < 0 or seed > 2**32 - 1:
                         raise ValueError(
                             'Argument "seed" must be an integer value'
                             ' between 0 and 2**32-1,'
@@ -861,7 +919,7 @@ class CmdStanArgs:
             raise ValueError('Argument "data" must be string or dict')
 
         if self.inits is not None:
-            if isinstance(self.inits, (float, int)):
+            if isinstance(self.inits, (float, int, np.floating, np.integer)):
                 if self.inits < 0:
                     raise ValueError(
                         'Argument "inits" must be > 0, found {}'.format(
@@ -869,8 +927,13 @@ class CmdStanArgs:
                         )
                     )
             elif isinstance(self.inits, str):
-                if not os.path.exists(self.inits):
-                    raise ValueError('no such file {}'.format(self.inits))
+                if not (
+                    isinstance(self.method_args, SamplerArgs)
+                    and self.method_args.num_chains > 1
+                    or isinstance(self.method_args, PathfinderArgs)
+                ):
+                    if not os.path.exists(self.inits):
+                        raise ValueError('no such file {}'.format(self.inits))
             elif isinstance(self.inits, list):
                 if self.chain_ids is None:
                     raise ValueError(
@@ -896,7 +959,6 @@ class CmdStanArgs:
         *,
         diagnostic_file: Optional[str] = None,
         profile_file: Optional[str] = None,
-        num_chains: Optional[int] = None
     ) -> List[str]:
         """
         Compose CmdStan command for non-default arguments.
@@ -910,36 +972,34 @@ class CmdStanArgs:
                     )
                 )
             cmd.append(self.model_exe)  # type: ignore # guaranteed by validate
-            cmd.append('id={}'.format(self.chain_ids[idx]))
+            cmd.append(f'id={self.chain_ids[idx]}')
         else:
             cmd.append(self.model_exe)  # type: ignore # guaranteed by validate
 
         if self.seed is not None:
             if not isinstance(self.seed, list):
                 cmd.append('random')
-                cmd.append('seed={}'.format(self.seed))
+                cmd.append(f'seed={self.seed}')
             else:
                 cmd.append('random')
-                cmd.append('seed={}'.format(self.seed[idx]))
+                cmd.append(f'seed={self.seed[idx]}')
         if self.data is not None:
             cmd.append('data')
-            cmd.append('file={}'.format(self.data))
+            cmd.append(f'file={self.data}')
         if self.inits is not None:
             if not isinstance(self.inits, list):
-                cmd.append('init={}'.format(self.inits))
+                cmd.append(f'init={self.inits}')
             else:
-                cmd.append('init={}'.format(self.inits[idx]))
+                cmd.append(f'init={self.inits[idx]}')
         cmd.append('output')
-        cmd.append('file={}'.format(csv_file))
+        cmd.append(f'file={csv_file}')
         if diagnostic_file:
-            cmd.append('diagnostic_file={}'.format(diagnostic_file))
+            cmd.append(f'diagnostic_file={diagnostic_file}')
         if profile_file:
-            cmd.append('profile_file={}'.format(profile_file))
+            cmd.append(f'profile_file={profile_file}')
         if self.refresh is not None:
-            cmd.append('refresh={}'.format(self.refresh))
+            cmd.append(f'refresh={self.refresh}')
         if self.sig_figs is not None:
-            cmd.append('sig_figs={}'.format(self.sig_figs))
+            cmd.append(f'sig_figs={self.sig_figs}')
         cmd = self.method_args.compose(idx, cmd)
-        if num_chains:
-            cmd.append('num_chains={}'.format(num_chains))
         return cmd
