@@ -22,6 +22,7 @@ import json
 import os
 import platform
 import re
+import shutil
 import sys
 import tarfile
 import urllib.error
@@ -41,6 +42,7 @@ from cmdstanpy.utils import (
     validate_dir,
     wrap_url_progress_hook,
 )
+from cmdstanpy.utils.cmdstan import get_download_url
 
 from . import progress as progbar
 
@@ -316,7 +318,7 @@ def _wrap_build_progress_hook() -> Optional[Callable[[str], None]]:
     pbar: tqdm = tqdm(
         total=msgs_expected,
         bar_format="{desc} ({elapsed}) | {bar} | {postfix[0][value]}",
-        postfix=[dict(value=f'Building CmdStan {pad}')],
+        postfix=[{"value": f'Building CmdStan {pad}'}],
         colour='blue',
         desc='',
         position=0,
@@ -347,12 +349,11 @@ def compile_example(verbose: bool = False) -> None:
 
     :param verbose: Boolean value; when ``True``, show output from make command.
     """
-    cmd = [
-        MAKE,
-        Path(
-            os.path.join('examples', 'bernoulli', 'bernoulli' + EXTENSION)
-        ).as_posix(),
-    ]
+    path = Path('examples', 'bernoulli', 'bernoulli').with_suffix(EXTENSION)
+    if path.is_file():
+        path.unlink()
+
+    cmd = [MAKE, path.as_posix()]
     try:
         if verbose:
             do_command(cmd)
@@ -360,7 +361,10 @@ def compile_example(verbose: bool = False) -> None:
             do_command(cmd, fd_out=None)
     except RuntimeError as e:
         # pylint: disable=raise-missing-from
-        raise CmdStanInstallError(f'Command "make clean-all" failed\n{e}')
+        raise CmdStanInstallError(f'Command "{" ".join(cmd)}" failed:\n{e}')
+
+    if not path.is_file():
+        raise CmdStanInstallError("Failed to generate example binary")
 
 
 def rebuild_cmdstan(
@@ -419,17 +423,15 @@ def install_version(
             clean_all(verbose)
             print('Rebuilding version {}'.format(cmdstan_version))
         build(verbose, progress=progress, cores=cores)
-        print('Test model compilation')
-        compile_example(verbose)
     print('Installed {}'.format(cmdstan_version))
 
 
 def is_version_available(version: str) -> bool:
+    if 'git:' in version:
+        return True  # no good way in general to check if a git tag exists
+
     is_available = True
-    url = (
-        'https://github.com/stan-dev/cmdstan/releases/download/'
-        'v{0}/cmdstan-{0}.tar.gz'.format(version)
-    )
+    url = get_download_url(version)
     for i in range(6):
         try:
             urllib.request.urlopen(url)
@@ -457,11 +459,29 @@ def retrieve_version(version: str, progress: bool = True) -> None:
     """Download specified CmdStan version."""
     if version is None or version == '':
         raise ValueError('Argument "version" unspecified.')
+
+    if 'git:' in version:
+        tag = version.split(':')[1]
+        tag_folder = version.replace(':', '-').replace('/', '_')
+        print(f"Cloning CmdStan branch '{tag}' from stan-dev/cmdstan on GitHub")
+        do_command(
+            [
+                'git',
+                'clone',
+                '--depth',
+                '1',
+                '--branch',
+                tag,
+                '--recursive',
+                '--shallow-submodules',
+                'https://github.com/stan-dev/cmdstan.git',
+                f'cmdstan-{tag_folder}',
+            ]
+        )
+        return
+
     print('Downloading CmdStan version {}'.format(version))
-    url = (
-        'https://github.com/stan-dev/cmdstan/releases/download/'
-        'v{0}/cmdstan-{0}.tar.gz'.format(version)
-    )
+    url = get_download_url(version)
     for i in range(6):  # always retry to allow for transient URLErrors
         try:
             if progress and progbar.allow_show_progress():
@@ -503,6 +523,8 @@ def retrieve_version(version: str, progress: bool = True) -> None:
         first = tar.next()
         if first is not None:
             top_dir = first.name
+        else:
+            top_dir = ''
         cmdstan_dir = f'cmdstan-{version}'
         if top_dir != cmdstan_dir:
             raise CmdStanInstallError(
@@ -573,16 +595,8 @@ def run_install(args: Union[InteractiveSettings, InstallationSettings]) -> None:
     """
     Run a (potentially interactive) installation
     """
-    if is_version_available(args.version):
-        print('Installing CmdStan version: {}'.format(args.version))
-    else:
-        raise ValueError(
-            f'Version {args.version} cannot be downloaded. '
-            'Connection to GitHub failed. '
-            'Check firewall settings or ensure this version exists.'
-        )
     validate_dir(args.dir)
-    print('Install directory: {}'.format(args.dir))
+    print('CmdStan install directory: {}'.format(args.dir))
 
     # these accesses just 'warm up' the interactive install
     _ = args.progress
@@ -591,9 +605,12 @@ def run_install(args: Union[InteractiveSettings, InstallationSettings]) -> None:
     if args.compiler:
         run_compiler_install(args.dir, args.verbose, args.progress)
 
-    cmdstan_version = f'cmdstan-{args.version}'
+    if 'git:' in args.version:
+        tag = args.version.replace(':', '-').replace('/', '_')
+        cmdstan_version = f'cmdstan-{tag}'
+    else:
+        cmdstan_version = f'cmdstan-{args.version}'
     with pushd(args.dir):
-
         already_installed = os.path.exists(cmdstan_version) and os.path.exists(
             os.path.join(
                 cmdstan_version,
@@ -603,6 +620,15 @@ def run_install(args: Union[InteractiveSettings, InstallationSettings]) -> None:
             )
         )
         if not already_installed or args.overwrite:
+            if is_version_available(args.version):
+                print('Installing CmdStan version: {}'.format(args.version))
+            else:
+                raise ValueError(
+                    f'Version {args.version} cannot be downloaded. '
+                    'Connection to GitHub failed. '
+                    'Check firewall settings or ensure this version exists.'
+                )
+            shutil.rmtree(cmdstan_version, ignore_errors=True)
             retrieve_version(args.version, args.progress)
             install_version(
                 cmdstan_version=cmdstan_version,
@@ -614,6 +640,10 @@ def run_install(args: Union[InteractiveSettings, InstallationSettings]) -> None:
         else:
             print('CmdStan version {} already installed'.format(args.version))
 
+        with pushd(cmdstan_version):
+            print('Test model compilation')
+            compile_example(args.verbose)
+
 
 def parse_cmdline_args() -> Dict[str, Any]:
     parser = argparse.ArgumentParser("install_cmdstan")
@@ -625,7 +655,11 @@ def parse_cmdline_args() -> Dict[str, Any]:
         + "interactive mode",
     )
     parser.add_argument(
-        '--version', '-v', help="version, defaults to latest release version"
+        '--version',
+        '-v',
+        help="version, defaults to latest release version. "
+        "If git is installed, you can also specify a git tag or branch, "
+        "e.g. git:develop",
     )
     parser.add_argument(
         '--dir', '-d', help="install directory, defaults to '$HOME/.cmdstan"
