@@ -31,13 +31,8 @@ except ImportError:
 
 
 from cmdstanpy.cmdstan_args import Method
-from cmdstanpy.utils import (
-    BaseType,
-    build_xarray_data,
-    flatten_chains,
-    get_logger,
-    scan_generated_quantities_csv,
-)
+from cmdstanpy.utils import build_xarray_data, flatten_chains, get_logger
+from cmdstanpy.utils.stancsv import scan_generic_csv
 
 from .mcmc import CmdStanMCMC
 from .metadata import InferenceMetadata
@@ -114,11 +109,11 @@ class CmdStanGQ(Generic[Fit]):
         dzero = {}
         for i in range(self.chains):
             if i == 0:
-                dzero = scan_generated_quantities_csv(
+                dzero = scan_generic_csv(
                     path=self.runset.csv_files[i],
                 )
             else:
-                drest = scan_generated_quantities_csv(
+                drest = scan_generic_csv(
                     path=self.runset.csv_files[i],
                 )
                 for key in dzero:
@@ -177,6 +172,7 @@ class CmdStanGQ(Generic[Fit]):
         self,
         *,
         inc_warmup: bool = False,
+        inc_iterations: bool = False,
         concat_chains: bool = False,
         inc_sample: bool = False,
     ) -> np.ndarray:
@@ -209,6 +205,7 @@ class CmdStanGQ(Generic[Fit]):
         CmdStanMCMC.draws
         """
         self._assemble_generated_quantities()
+        inc_warmup |= inc_iterations
         if inc_warmup:
             if (
                 isinstance(self.previous_fit, CmdStanMCMC)
@@ -242,7 +239,9 @@ class CmdStanGQ(Generic[Fit]):
             ]
             drop_cols: List[int] = []
             for dup in dups:
-                drop_cols.extend(self.previous_fit.metadata.stan_vars_cols[dup])
+                drop_cols.extend(
+                    self.previous_fit._metadata.stan_vars[dup].columns()
+                )
 
         start_idx, _ = self._draws_start(inc_warmup)
         previous_draws = self._previous_draws(True)
@@ -324,45 +323,75 @@ class CmdStanGQ(Generic[Fit]):
 
         self._assemble_generated_quantities()
 
-        gq_cols = []
-        mcmc_vars = []
+        all_columns = ['chain__', 'iter__', 'draw__'] + list(self.column_names)
+
+        gq_cols: List[str] = []
+        mcmc_vars: List[str] = []
         if vars is not None:
             for var in vars_list:
-                if var in self.metadata.stan_vars_cols:
-                    for idx in self.metadata.stan_vars_cols[var]:
-                        gq_cols.append(self.column_names[idx])
+                if var in self._metadata.stan_vars:
+                    info = self._metadata.stan_vars[var]
+                    gq_cols.extend(
+                        self.column_names[info.start_idx : info.end_idx]
+                    )
                 elif (
-                    inc_sample
-                    and var in self.previous_fit.metadata.stan_vars_cols
+                    inc_sample and var in self.previous_fit._metadata.stan_vars
                 ):
-                    mcmc_vars.append(var)
+                    info = self.previous_fit._metadata.stan_vars[var]
+                    mcmc_vars.extend(
+                        self.previous_fit.column_names[
+                            info.start_idx : info.end_idx
+                        ]
+                    )
+                elif var in ['chain__', 'iter__', 'draw__']:
+                    gq_cols.append(var)
                 else:
                     raise ValueError('Unknown variable: {}'.format(var))
         else:
-            gq_cols = list(self.column_names)
+            gq_cols = all_columns
             vars_list = gq_cols
 
         previous_draws_pd = self._previous_draws_pd(mcmc_vars, inc_warmup)
 
+        draws = self.draws(inc_warmup=inc_warmup)
+        # add long-form columns for chain, iteration, draw
+        n_draws, n_chains, _ = draws.shape
+        chains_col = (
+            np.repeat(np.arange(1, n_chains + 1), n_draws)
+            .reshape(1, n_chains, n_draws)
+            .T
+        )
+        iter_col = (
+            np.tile(np.arange(1, n_draws + 1), n_chains)
+            .reshape(1, n_chains, n_draws)
+            .T
+        )
+        draw_col = (
+            np.arange(1, (n_draws * n_chains) + 1)
+            .reshape(1, n_chains, n_draws)
+            .T
+        )
+        draws = np.concatenate([chains_col, iter_col, draw_col, draws], axis=2)
+
+        draws_pd = pd.DataFrame(
+            data=flatten_chains(draws),
+            columns=all_columns,
+        )
+
         if inc_sample and mcmc_vars:
             if gq_cols:
                 return pd.concat(
                     [
                         previous_draws_pd,
-                        pd.DataFrame(
-                            data=flatten_chains(
-                                self.draws(inc_warmup=inc_warmup)
-                            ),
-                            columns=self.column_names,
-                        )[gq_cols],
+                        draws_pd[gq_cols],
                     ],
                     axis='columns',
                 )[vars_list]
             else:
                 return previous_draws_pd
         elif inc_sample and vars is None:
-            cols_1 = self.previous_fit.column_names
-            cols_2 = self.column_names
+            cols_1 = list(previous_draws_pd.columns)
+            cols_2 = list(draws_pd.columns)
             dups = [
                 item
                 for item, count in Counter(cols_1 + cols_2).items()
@@ -371,23 +400,14 @@ class CmdStanGQ(Generic[Fit]):
             return pd.concat(
                 [
                     previous_draws_pd.drop(columns=dups).reset_index(drop=True),
-                    pd.DataFrame(
-                        data=flatten_chains(self.draws(inc_warmup=inc_warmup)),
-                        columns=self.column_names,
-                    ),
+                    draws_pd,
                 ],
                 axis=1,
             )
         elif gq_cols:
-            return pd.DataFrame(
-                data=flatten_chains(self.draws(inc_warmup=inc_warmup)),
-                columns=self.column_names,
-            )[gq_cols]
+            return draws_pd[gq_cols]
 
-        return pd.DataFrame(
-            data=flatten_chains(self.draws(inc_warmup=inc_warmup)),
-            columns=self.column_names,
-        )
+        return draws_pd
 
     @overload
     def draws_xr(
@@ -448,18 +468,18 @@ class CmdStanGQ(Generic[Fit]):
             else:
                 vars_list = vars
             for var in vars_list:
-                if var not in self.metadata.stan_vars_cols:
+                if var not in self._metadata.stan_vars:
                     if inc_sample and (
-                        var in self.previous_fit.metadata.stan_vars_cols
+                        var in self.previous_fit._metadata.stan_vars
                     ):
                         mcmc_vars_list.append(var)
                         dup_vars.append(var)
                     else:
                         raise ValueError('Unknown variable: {}'.format(var))
         else:
-            vars_list = list(self.metadata.stan_vars_cols.keys())
+            vars_list = list(self._metadata.stan_vars.keys())
             if inc_sample:
-                for var in self.previous_fit.metadata.stan_vars_cols.keys():
+                for var in self.previous_fit._metadata.stan_vars.keys():
                     if var not in vars_list and var not in mcmc_vars_list:
                         mcmc_vars_list.append(var)
         for var in dup_vars:
@@ -468,7 +488,7 @@ class CmdStanGQ(Generic[Fit]):
         self._assemble_generated_quantities()
 
         num_draws = self.previous_fit.num_draws_sampling
-        sample_config = self.previous_fit.metadata.cmdstan_config
+        sample_config = self.previous_fit._metadata.cmdstan_config
         attrs: MutableMapping[Hashable, Any] = {
             "stan_version": f"{sample_config['stan_version_major']}."
             f"{sample_config['stan_version_minor']}."
@@ -489,34 +509,22 @@ class CmdStanGQ(Generic[Fit]):
         for var in vars_list:
             build_xarray_data(
                 data,
-                var,
-                self._metadata.stan_vars_dims[var],
-                self._metadata.stan_vars_cols[var],
-                0,
+                self._metadata.stan_vars[var],
                 self.draws(inc_warmup=inc_warmup),
-                self._metadata.stan_vars_types[var],
             )
         if inc_sample:
             for var in mcmc_vars_list:
                 build_xarray_data(
                     data,
-                    var,
-                    self.previous_fit.metadata.stan_vars_dims[var],
-                    self.previous_fit.metadata.stan_vars_cols[var],
-                    0,
+                    self.previous_fit._metadata.stan_vars[var],
                     self.previous_fit.draws(inc_warmup=inc_warmup),
-                    self.previous_fit._metadata.stan_vars_types[var],
                 )
 
         return xr.Dataset(data, coords=coordinates, attrs=attrs).transpose(
             'chain', 'draw', ...
         )
 
-    def stan_variable(
-        self,
-        var: str,
-        inc_warmup: bool = False,
-    ) -> np.ndarray:
+    def stan_variable(self, var: str, **kwargs: bool) -> np.ndarray:
         """
         Return a numpy.ndarray which contains the set of draws
         for the named Stan program variable.  Flattens the chains,
@@ -530,13 +538,13 @@ class CmdStanGQ(Generic[Fit]):
         the next M are from chain 2, and the last M elements are from chain N.
 
         * If the variable is a scalar variable, the return array has shape
-          ( draws X chains, 1).
+          ( draws * chains, 1).
         * If the variable is a vector, the return array has shape
-          ( draws X chains, len(vector))
+          ( draws * chains, len(vector))
         * If the variable is a matrix, the return array has shape
-          ( draws X chains, size(dim 1) X size(dim 2) )
+          ( draws * chains, size(dim 1), size(dim 2) )
         * If the variable is an array with N dimensions, the return array
-          has shape ( draws X chains, size(dim 1) X ... X size(dim N))
+          has shape ( draws * chains, size(dim 1), ..., size(dim N))
 
         For example, if the Stan program variable ``theta`` is a 3x3 matrix,
         and the sample consists of 4 chains with 1000 post-warmup draws,
@@ -547,19 +555,21 @@ class CmdStanGQ(Generic[Fit]):
 
         :param var: variable name
 
-        :param inc_warmup: When ``True`` and the warmup draws are present in
-            the MCMC sample, then the warmup draws are included.
-            Default value is ``False``.
+        :param kwargs: Additional keyword arguments are passed to the underlying
+            fit's ``stan_variable`` method if the variable is not a generated
+            quantity.
 
         See Also
         --------
         CmdStanGQ.stan_variables
         CmdStanMCMC.stan_variable
         CmdStanMLE.stan_variable
+        CmdStanPathfinder.stan_variable
         CmdStanVB.stan_variable
+        CmdStanLaplace.stan_variable
         """
-        model_var_names = self.previous_fit.metadata.stan_vars_cols.keys()
-        gq_var_names = self.metadata.stan_vars_cols.keys()
+        model_var_names = self.previous_fit._metadata.stan_vars.keys()
+        gq_var_names = self._metadata.stan_vars.keys()
         if not (var in model_var_names or var in gq_var_names):
             raise ValueError(
                 f'Unknown variable name: {var}\n'
@@ -567,65 +577,48 @@ class CmdStanGQ(Generic[Fit]):
                 + ", ".join(model_var_names | gq_var_names)
             )
         if var not in gq_var_names:
-            if isinstance(self.previous_fit, CmdStanMCMC):
-                return self.previous_fit.stan_variable(
-                    var, inc_warmup=inc_warmup
-                )
-            elif isinstance(self.previous_fit, CmdStanMLE):
-                return np.atleast_1d(  # type: ignore
-                    np.asarray(
-                        self.previous_fit.stan_variable(
-                            var, inc_iterations=inc_warmup
-                        )
-                    )
-                )
-            else:
-                return np.atleast_1d(  # type: ignore
-                    np.asarray(self.previous_fit.stan_variable(var))
-                )
+            # TODO(2.0) atleast1d may not be needed
+            return np.atleast_1d(  # type: ignore
+                self.previous_fit.stan_variable(var, **kwargs)
+            )
 
         # is gq variable
         self._assemble_generated_quantities()
 
-        draw1, num_draws = self._draws_start(inc_warmup)
-        dims = [num_draws * self.chains]
-        col_idxs = self._metadata.stan_vars_cols[var]
-        if len(col_idxs) > 0:
-            dims.extend(self._metadata.stan_vars_dims[var])
-        draws = self._draws[draw1:, :, col_idxs]
+        draw1, _ = self._draws_start(
+            inc_warmup=kwargs.get('inc_warmup', False)
+            or kwargs.get('inc_iterations', False)
+        )
+        draws = flatten_chains(self._draws[draw1:])
+        out: np.ndarray = self._metadata.stan_vars[var].extract_reshape(draws)
+        return out
 
-        if self._metadata.stan_vars_types[var] == BaseType.COMPLEX:
-            draws = draws[..., ::2] + 1j * draws[..., 1::2]
-            dims = dims[:-1]
-
-        draws = draws.reshape(dims, order='F')
-
-        return draws
-
-    def stan_variables(self, inc_warmup: bool = False) -> Dict[str, np.ndarray]:
+    def stan_variables(self, **kwargs: bool) -> Dict[str, np.ndarray]:
         """
         Return a dictionary mapping Stan program variables names
         to the corresponding numpy.ndarray containing the inferred values.
 
-        :param inc_warmup: When ``True`` and the warmup draws are present in
-            the MCMC sample, then the warmup draws are included.
-            Default value is ``False``
+        :param kwargs: Additional keyword arguments are passed to the underlying
+            fit's ``stan_variable`` method if the variable is not a generated
+            quantity.
 
         See Also
         --------
         CmdStanGQ.stan_variable
         CmdStanMCMC.stan_variables
         CmdStanMLE.stan_variables
+        CmdStanPathfinder.stan_variables
         CmdStanVB.stan_variables
+        CmdStanLaplace.stan_variables
         """
         result = {}
-        sample_var_names = self.previous_fit.metadata.stan_vars_cols.keys()
-        gq_var_names = self.metadata.stan_vars_cols.keys()
+        sample_var_names = self.previous_fit._metadata.stan_vars.keys()
+        gq_var_names = self._metadata.stan_vars.keys()
         for name in gq_var_names:
-            result[name] = self.stan_variable(name, inc_warmup)
+            result[name] = self.stan_variable(name, **kwargs)
         for name in sample_var_names:
             if name not in gq_var_names:
-                result[name] = self.stan_variable(name, inc_warmup)
+                result[name] = self.stan_variable(name, **kwargs)
         return result
 
     def _assemble_generated_quantities(self) -> None:
@@ -661,7 +654,6 @@ class CmdStanGQ(Generic[Fit]):
         elif isinstance(p_fit, CmdStanMLE):
             num_draws = 1
             if p_fit._save_iterations:
-
                 opt_iters = len(p_fit.optimized_iterations_np)  # type: ignore
                 if inc_warmup:
                     num_draws = opt_iters
@@ -687,9 +679,9 @@ class CmdStanGQ(Generic[Fit]):
             if inc_warmup and p_fit._save_iterations:
                 return p_fit.optimized_iterations_np[:, None]  # type: ignore
 
-            return np.atleast_2d(p_fit.optimized_params_np,)[  # type: ignore
-                :, None
-            ]
+            return np.atleast_2d(  # type: ignore
+                p_fit.optimized_params_np,
+            )[:, None]
         else:  # CmdStanVB:
             if inc_warmup:
                 return np.vstack(
@@ -710,7 +702,6 @@ class CmdStanGQ(Generic[Fit]):
             return p_fit.draws_pd(vars or None, inc_warmup=inc_warmup)
 
         elif isinstance(p_fit, CmdStanMLE):
-
             if inc_warmup and p_fit._save_iterations:
                 return p_fit.optimized_iterations_pd[sel]  # type: ignore
             else:
@@ -734,6 +725,7 @@ class CmdStanGQ(Generic[Fit]):
         """
         self.runset.save_csvfiles(dir)
 
+    # TODO(2.0): remove
     @property
     def mcmc_sample(self) -> Union[CmdStanMCMC, CmdStanMLE, CmdStanVB]:
         get_logger().warning(
