@@ -38,8 +38,8 @@ def determine_linux_arch() -> str:
             arch = "armhf"
     elif machine == "mips64":
         arch = "mips64el"
-    elif machine == "ppc64el":
-        arch = "ppc64le"
+    elif machine == "ppc64el" or machine == "ppc64le":
+        arch = "ppc64el"
     elif machine == "s390x":
         arch = "s390x"
     return arch
@@ -97,11 +97,13 @@ def get_latest_cmdstan(cmdstan_dir: str) -> Optional[str]:
         for name in os.listdir(cmdstan_dir)
         if os.path.isdir(os.path.join(cmdstan_dir, name))
         and name.startswith('cmdstan-')
-        and name[8].isdigit()
-        and len(name[8:].split('.')) == 3
     ]
     if len(versions) == 0:
         return None
+    if len(versions) == 1:
+        return 'cmdstan-' + versions[0]
+    # we can only compare numeric versions
+    versions = [v for v in versions if v[0].isdigit() and v.count('.') == 2]
     # munge rc for sort, e.g. 2.25.0-rc1 -> 2.25.-99
     for i in range(len(versions)):  # # pylint: disable=C0200
         if '-rc' in versions[i]:
@@ -280,7 +282,7 @@ def cxx_toolchain_path(
         if os.path.exists(os.path.join(toolchain_root, 'mingw64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
+                'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -304,7 +306,7 @@ def cxx_toolchain_path(
         elif os.path.exists(os.path.join(toolchain_root, 'mingw_64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
+                'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -356,7 +358,7 @@ def cxx_toolchain_path(
                 if version not in ('35', '3.5', '3'):
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
+                        'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -381,7 +383,7 @@ def cxx_toolchain_path(
                 else:
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
+                        'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -442,6 +444,8 @@ def install_cmdstan(
 
     :param version: CmdStan version string, e.g. "2.29.2".
         Defaults to latest CmdStan release.
+        If ``git`` is installed, a git tag or branch of stan-dev/cmdstan
+        can be specified, e.g. "git:develop".
 
     :param dir: Path to install directory.  Defaults to hidden directory
         ``$HOME/.cmdstan``.
@@ -516,7 +520,11 @@ def install_cmdstan(
         logger.warning('CmdStan installation failed.\n%s', str(e))
         return False
 
-    set_cmdstan_path(os.path.join(args.dir, f"cmdstan-{args.version}"))
+    if 'git:' in args.version:
+        folder = f"cmdstan-{args.version.replace(':', '-').replace('/', '_')}"
+    else:
+        folder = f"cmdstan-{args.version}"
+    set_cmdstan_path(os.path.join(args.dir, folder))
 
     return True
 
