@@ -4,8 +4,7 @@ Common functions for reshaping numpy arrays
 from typing import Hashable, MutableMapping, Tuple
 
 import numpy as np
-
-from .stancsv import BaseType
+import stanio
 
 
 def flatten_chains(draws_array: np.ndarray) -> np.ndarray:
@@ -29,37 +28,17 @@ def flatten_chains(draws_array: np.ndarray) -> np.ndarray:
 
 def build_xarray_data(
     data: MutableMapping[Hashable, Tuple[Tuple[str, ...], np.ndarray]],
-    var_name: str,
-    dims: Tuple[int, ...],
-    col_idxs: Tuple[int, ...],
-    start_row: int,
+    var: stanio.Variable,
     drawset: np.ndarray,
-    var_type: BaseType,
 ) -> None:
     """
     Adds Stan variable name, labels, and values to a dictionary
     that will be used to construct an xarray DataSet.
     """
     var_dims: Tuple[str, ...] = ('draw', 'chain')
-    if dims:
-        var_dims += tuple(f"{var_name}_dim_{i}" for i in range(len(dims)))
+    var_dims += tuple(f"{var.name}_dim_{i}" for i in range(len(var.dimensions)))
 
-        draws = drawset[start_row:, :, col_idxs]
-
-        if var_type == BaseType.COMPLEX:
-            draws = draws[..., ::2] + 1j * draws[..., 1::2]
-            var_dims = var_dims[:-1]
-            dims = dims[:-1]
-
-        draws = draws.reshape(*drawset.shape[:2], *dims, order="F")
-
-        data[var_name] = (
-            var_dims,
-            draws,
-        )
-
-    else:
-        data[var_name] = (
-            var_dims,
-            np.squeeze(drawset[start_row:, :, col_idxs], axis=2),
-        )
+    data[var.name] = (
+        var_dims,
+        var.extract_reshape(drawset),
+    )
