@@ -1,14 +1,14 @@
 """CmdStanModel tests"""
 
 import contextlib
-from glob import glob
 import io
 import logging
 import os
 import re
 import shutil
 import tempfile
-from test import raises_nested, check_present
+from glob import glob
+from test import check_present, raises_nested
 from typing import List
 from unittest.mock import MagicMock, patch
 
@@ -137,7 +137,6 @@ def test_model_bad() -> None:
 
 
 def test_stanc_options() -> None:
-
     allowed_optims = ("", "0", "1", "experimental")
     for optim in allowed_optims:
         opts = {
@@ -282,7 +281,7 @@ def test_compile_with_includes(
     # Compile after modifying included file, ensuring cache is not used.
     def _patched_getmtime(filename: str) -> float:
         includes = ['divide_real_by_two.stan', 'add_one_function.stan']
-        if any(filename.endswith(include) for include in includes):
+        if any(str(filename).endswith(include) for include in includes):
             return float('inf')
         return getmtime(filename)
 
@@ -323,8 +322,7 @@ def test_compile_force() -> None:
     model.compile(force=True, cpp_options=override_opts, override_options=True)
     info_dict3 = model.exe_info()
     assert info_dict3['STAN_THREADS'].lower() == 'false'
-    # cmdstan#1056
-    # assert info_dict3['STAN_NO_RANGE_CHECKS'].lower() == 'true'
+    assert info_dict3['STAN_NO_RANGE_CHECKS'].lower() == 'true'
 
     model.compile(force=True, cpp_options=more_opts)
     info_dict4 = model.exe_info()
@@ -426,16 +424,19 @@ def test_model_compile() -> None:
     assert exe_time == os.path.getmtime(model2.exe_file)
 
 
-def test_model_compile_space() -> None:
+@pytest.mark.parametrize("path", ["space in path", "tilde~in~path"])
+def test_model_compile_special_char(path: str) -> None:
     with tempfile.TemporaryDirectory(
         prefix="cmdstanpy_testfolder_"
     ) as tmp_path:
-        path_with_space = os.path.join(tmp_path, "space in path")
-        os.makedirs(path_with_space, exist_ok=True)
+        path_with_special_char = os.path.join(tmp_path, path)
+        os.makedirs(path_with_special_char, exist_ok=True)
         bern_stan_new = os.path.join(
-            path_with_space, os.path.split(BERN_STAN)[1]
+            path_with_special_char, os.path.split(BERN_STAN)[1]
+        )
+        bern_exe_new = os.path.join(
+            path_with_special_char, os.path.split(BERN_EXE)[1]
         )
-        bern_exe_new = os.path.join(path_with_space, os.path.split(BERN_EXE)[1])
         shutil.copyfile(BERN_STAN, bern_stan_new)
         model = CmdStanModel(stan_file=bern_stan_new)
 
@@ -451,6 +452,38 @@ def test_model_compile_space() -> None:
         assert exe_time == os.path.getmtime(model2.exe_file)
 
 
+@pytest.mark.parametrize("path", ["space in path", "tilde~in~path"])
+def test_model_includes_special_char(path: str) -> None:
+    """Test model with include file in path with spaces."""
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli_include.stan')
+    stan_divide = os.path.join(DATAFILES_PATH, 'divide_real_by_two.stan')
+
+    with tempfile.TemporaryDirectory(
+        prefix="cmdstanpy_testfolder_"
+    ) as tmp_path:
+        path_with_special_char = os.path.join(tmp_path, path)
+        os.makedirs(path_with_special_char, exist_ok=True)
+        bern_stan_new = os.path.join(
+            path_with_special_char, os.path.split(stan)[1]
+        )
+        stan_divide_new = os.path.join(
+            path_with_special_char, os.path.split(stan_divide)[1]
+        )
+        shutil.copyfile(stan, bern_stan_new)
+        shutil.copyfile(stan_divide, stan_divide_new)
+
+        model = CmdStanModel(
+            stan_file=bern_stan_new,
+            stanc_options={'include-paths': path_with_special_char},
+        )
+        assert path in str(model.exe_file)
+
+        assert path in model.src_info()['included_files'][0]
+        assert (
+            "divide_real_by_two.stan" in model.src_info()['included_files'][0]
+        )
+
+
 def test_model_includes_explicit() -> None:
     if os.path.exists(BERN_EXE):
         os.remove(BERN_EXE)
@@ -461,6 +494,18 @@ def test_model_includes_explicit() -> None:
     assert os.path.samefile(model.exe_file, BERN_EXE)
 
 
+def test_model_compile_with_explicit_includes() -> None:
+    stan_file = os.path.join(DATAFILES_PATH, "add_one_model.stan")
+    exe_file = os.path.splitext(stan_file)[0] + EXTENSION
+    if os.path.isfile(exe_file):
+        os.unlink(exe_file)
+
+    model = CmdStanModel(stan_file=stan_file, compile=False)
+    include_paths = [os.path.join(DATAFILES_PATH, "include-path")]
+    stanc_options = {"include-paths": include_paths}
+    model.compile(stanc_options=stanc_options)
+
+
 def test_model_includes_implicit() -> None:
     stan = os.path.join(DATAFILES_PATH, 'bernoulli_include.stan')
     exe = os.path.join(DATAFILES_PATH, 'bernoulli_include' + EXTENSION)
@@ -481,19 +526,11 @@ def test_model_format_deprecations() -> None:
 
     sys_stdout = io.StringIO()
     with contextlib.redirect_stdout(sys_stdout):
-        model.format()
+        model.format(canonicalize=True)
 
     formatted = sys_stdout.getvalue()
     assert "//" in formatted
     assert "#" not in formatted
-    assert formatted.count('(') == 5
-
-    sys_stdout = io.StringIO()
-    with contextlib.redirect_stdout(sys_stdout):
-        model.format(canonicalize=True)
-
-    formatted = sys_stdout.getvalue()
-    print(formatted)
     assert "<-" not in formatted
     assert formatted.count('(') == 0
 
